# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                         CREATE BASEMAPS                          #
#                                                                  #
####################################################################

This script creates the final AWI Basemaps in all three projections
and with different color palettes. This happens in two steps. First
the DEM is rendered with the color palette into an RGB color raster.
Then the shaded map is calculated by combining the colored raster
with the shading raster.
At the end, the raster files are finalized by creating tiles and
pyramids and by adding metadata tags.

"""

# ============================================================================
#    SCRIPT INFO
# ============================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.5'
__date__ = '2024-09-09'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Development'

# ============================================================================
#    IMPORT
# ============================================================================

import textwrap

# import rasterio

# import AWI Basemap configuration
from config import *

# ============================================================================
#    GLOBAL VARIABLES
# ============================================================================

# GDAL creation options for final result
GDAL_CREATE_OPTIONS_FINAL = '-co TILED=YES -co BLOCKXSIZE=512 -co BLOCKYSIZE=512 -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=NONE -co BIGTIFF=IF_NEEDED'

# Method to create overviews
OVERVIEW_METHOD = 'cubic'


# ============================================================================
#    FUNCTIONS
# ============================================================================

def create_basemap(name, grid, hillshade, color_palette, epsg, output):
    """Create the AWI Basemap RGB grid from various inputs

    Parameters
    ----------
    name : str
        Name of grid version
    grid : str
        Path to input DEM grid
    hillshade : str
        Path to input hillshade grid
    color_palette : str
        Path to input color palette file
    epsg : int
        EPSG code
    output : str
        Path to output RGB grid

    """
    # create RGB version with grid and color palette
    msg = f'Creating {name} RGB...'
    cmd = (f'gdaldem'
           f' color-relief'
           f' {grid}'
           f' {color_palette}'
           f' {TMP_RGB}'
           f' -of GTiff'
           f' -b 1'
           f' -compute_edges'
           f' {GDAL_CREATE_OPTIONS}')
    run(msg, cmd)

    # compile shaded and colored RGB grid
    msg = f'Creating shaded {name} map...'
    cmd = (f'gdal_calc'
           f' --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*254)"'
           f' --format GTiff'
           f' --type Byte'
           f' -A {TMP_RGB}'
           f' --A_band 1'
           f' -B {hillshade}'
           f' --B_band 1'
           f' --allBands=A'
           f' --overwrite'
           f' --outfile {TMP_SHADED}')
    # {GDAL_CREATE_OPTIONS_PY}
    run(msg, cmd)

    # cleanup
    cleanup(TMP_RGB)

    # finalize grid with GDAL create options
    msg = f'Finalizing {name}...'
    cmd = (f'gdal_translate'
           f' -a_srs EPSG:{epsg}'
           f' -stats'
           f' -of GTiff'
           f' {GDAL_CREATE_OPTIONS_FINAL}'
           f' {TMP_SHADED} {output}')
    run(msg, cmd)

    # cleanup
    cleanup(TMP_SHADED)
    cleanup(f'{TMP_SHADED}.aux.xml')

    # create internal pyramid layers
    msg = f'Creating pyramids for {name}...'
    cmd = (f'gdaladdo'
           f' -r {OVERVIEW_METHOD}'
           f' --config COMPRESS_OVERVIEW NONE'
           f' --config INTERLEAVE_OVERVIEW PIXEL'
           f' --config BIGTIFF_OVERVIEW IF_NEEDED'
           f' --config GDAL_TIFF_OVR_BLOCKSIZE 512'
           f' {output}'
           f' 2 4 8 16 32 64')
    run(msg, cmd)

    # set metadata in GeoTIFF tags
    print('====================================================================')
    print()
    print(f'Updating Metadata for {name}...')
    METADATA['NAME'] = name
    METADATA['EPSG'] = epsg
    with rasterio.open(output, 'r+') as ds:
        # update metadata
        ds.update_tags(**METADATA)
    print()


# ============================================================================
#    PROCESS
# ============================================================================

# heading
print()
print('====================================================================')
print(f'{"CREATE BASEMAPS":^68}')

# create the different basemaps with map parameters from config.py
for kwargs in MAPS:
    create_basemap(**kwargs)

# cleanup
cleanup(GEBCO_WORLD)
cleanup(GEBCO_WORLD_HILLSHADE)
cleanup(GEBCO_ARCTIC)
cleanup(GEBCO_ARCTIC_HILLSHADE)
cleanup(GEBCO_ANTARCTIC)
cleanup(GEBCO_ANTARCTIC_HILLSHADE)

# ============================================================================
#    METADATA
# ============================================================================

# write metadata to text file
with open(METADATA_FILE, 'w') as f:
    f.write(f'####################################################################\n')
    f.write(f'#                                                                  #\n')
    f.write(f'#{TITLE:^66}#\n')
    f.write(f'#{VERSION:^66}#\n')
    f.write(f'#                                                                  #\n')
    f.write(f'####################################################################\n\n')
    for key, val in METADATA.items():
        if key not in ['VERSION', 'NAME', 'EPSG']:
            f.write(f'{key.replace("_", " ")}\n')
            f.write(f'--------------------------------------------------------------------\n')
            for line in val.splitlines():
                # f.write(textwrap.fill(line, width=68, initial_indent='', subsequent_indent='  '))
                f.write(textwrap.fill(line, width=69, initial_indent='', subsequent_indent=''))
                f.write('\n\n')
            f.write(f'\n')

# ============================================================================
#    SERVICE.CONF
# ============================================================================

# write metadata to text file
with open(SERVICE_CONF_FILE, 'w') as f:
    f.write(f'''[SERVICE]
maintainer = https://www.awi.de/en/about-us/service/computing-centre/data-products.html
online.resource = https://maps.awi.de/services/basemaps/gs/awi
title = AWI Basemap Service
abstract = The AWI Basemap is a global basemap intended to be used as a
  background for GIS applications or web map viewers like the DAM
  marine data portal. The map is a rendered and shaded RGB version
  of the current GEBCO grid with ice overlays from ADD, GLIMS and
  GIMP. It is available in three projections, EPSG:4326 for the
  global map as well as EPSG:3995 and EPSG:3031 for the polar
  stereographic versions.
fees = NONE
access constraints = CC-BY 4.0
keywords = Basemap,Bathymetry,Topography,GEBCO

[LAYER.ROOT]
title = AWI Basemaps
abstract = The AWI Basemap is a global basemap intended to be used as a
  background for GIS applications or web map viewers like the DAM
  marine data portal. The map is a rendered and shaded RGB version
  of the current GEBCO grid with ice overlays from ADD, GLIMS and
  GIMP.

[LAYER.world_<color>]
name = world_<color>
title = AWI Basemap World View (<color>)
abstract = AWI Basemap World in EPSG:4326 [WGS84 / Geographic] with global coverage.

[LAYER.arctic_<color>]
name = arctic_<color>
title = AWI Basemap Arctic View (<color>)
abstract = AWI Basemap Arctic in EPSG:3995 [WGS 84 / Arctic Polar Stereographic] from the North Pole to {ARCTIC_EXTENT_LAT:.0f}°N.

[LAYER.antarctic_<color>]
name = antarctic_<color>
title = AWI Basemap Antarctic View (<color>)
abstract = AWI Basemap Antarctic in EPSG:3031 [WGS 84 / Antarctic Polar Stereographic] from the South Pole to {-ANTARCTIC_EXTENT_LAT:.0f}°S.
''')
